-- Daten für alle Charaktere
groupData = {};

local SEND_MAX_LEVEL_PREFIX = "GXP_MaxLevelData";
local SEND_DATA_PREFIX = "GXP_SendData";
local PARTY_UNITS = {"party1", "party2", "party3", "party4"};

-- functions
local debug = false
function gxp_debug(...)
    if (not debug) then
        return;
    end

    local message = "";

    for i = 1, select("#", ...) do
		message = message .. " " .. (select(i, ...) or "nil");
	end

    DEFAULT_CHAT_FRAME:AddMessage("CUI DEBUG:"..(message or "nil"), 1.0, 1.0, 1.0, 1, 10);
end

function GroupXP_OnLoad(self)
    gxp_debug("start GroupXP");

    hooksecurefunc("MultiActionBar_Update", CleanUI_MultiActionBar_Update);

    -- register for addon messages
    C_ChatInfo.RegisterAddonMessagePrefix(SEND_MAX_LEVEL_PREFIX);
    C_ChatInfo.RegisterAddonMessagePrefix(SEND_DATA_PREFIX);

    -- Register Events
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
    self:RegisterEvent("ADDON_LOADED");
    self:RegisterEvent("VARIABLES_LOADED");
    
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("PLAYER_LEVEL_UP");
	self:RegisterEvent("PLAYER_XP_UPDATE");
	self:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN");
	self:RegisterEvent("CHAT_MSG_ADDON");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
    self:RegisterEvent("RAID_ROSTER_UPDATE");

    self:RegisterEvent("CHAT_MSG_ADDON");

    GroupXP_InitSlashCommandHandler();
end

function GroupXP_InitSlashCommandHandler()

    SLASH_NOTES1 = "/gxp";
    SLASH_NOTES2 = "/groupxp";
    SlashCmdList["NOTES"] = function(msg)
        GroupXP_SlashCommandHandler(msg);
    end
end

function GroupXP_SlashCommandHandler(msg)
    if (msg == "reload") then
        ReloadUI();
    elseif (msg == "update") then
        GroupXP_AskData()
        GroupXP_CheckData()
    end
end

function GroupXP_OnEvent(self, event, ...)
    gxp_debug("event", event);
    if (event == "CHAT_MSG_ADDON") then
        local prefix, message, type, sender = ...;
        GroupXP_ReceiveData(prefix, message, type, sender);
    elseif (event == "GROUP_ROSTER_UPDATE") then
        GroupXP_CheckData();
        GroupXP_SendData();
    else
        GroupXP_SendData();
    end
end

function CleanUI_MultiActionBar_Update()
    GroupXP_UpdateUI();
    GroupXP_UpdatePlayerData();
end

--[[function GroupXP_SendFakeData()
    gxp_debug("SEND FAKE DATA");

    local message1 = "Hugo;MAGE;103;50000;200000";
    local message2 = "Emil;WARRIOR;103;60000;200000";
    local message3 = "Max;DEATHKNIGHT;103;70000;200000";
    local message4 = "Emma;WARLOCK;103;80000;200000";

    C_ChatInfo.SendAddonMessage(SEND_DATA_PREFIX, message1, "WHISPER", "Jaianto");
    C_ChatInfo.SendAddonMessage(SEND_DATA_PREFIX, message2, "WHISPER", "Jaianto");
    C_ChatInfo.SendAddonMessage(SEND_DATA_PREFIX, message3, "WHISPER", "Jaianto");
    C_ChatInfo.SendAddonMessage(SEND_DATA_PREFIX, message4, "WHISPER", "Jaianto");

    local exhaustionStateID, exhaustionStateName, exhaustionStateMultiplier = GetRestState();

    gxp_debug("exhaustionStateID", exhaustionStateID)
    gxp_debug("exhaustionStateName", exhaustionStateName)
    gxp_debug("exhaustionStateMultiplier", exhaustionStateMultiplier)

    GroupXP_DrawData(1, {name = "Hugo", englishClass = "MAGE", exhaustionStateID = 2, exhaustionStateName = "Erholt", exhaustionStateMultiplier = 2, level = 103, xp = 52678, maxXp = 100000});
    GroupXP_DrawData(2, {name = "Emil", englishClass = "WARRIOR", exhaustionStateID = 2, exhaustionStateName = "Erholt", exhaustionStateMultiplier = 2, level = 103, xp = 67122, maxXp = 100000});
    GroupXP_DrawData(3, {name = "Max", englishClass = "DEATHKNIGHT", exhaustionStateID = 2, exhaustionStateName = "Erholt", exhaustionStateMultiplier = 2, level = 103, xp = 74567, maxXp = 100000});
    GroupXP_DrawData(4, {name = "Emma", englishClass = "WARLOCK", exhaustionStateID = 2, exhaustionStateName = "Erholt", exhaustionStateMultiplier = 2, level = 103, xp = 85455, maxXp = 100000});
end]]

function GroupXP_SendData()
    local name = UnitName("player");
    local _, englishClass = UnitClass("player");
    local level = UnitLevel("player");
    local xp = UnitXP("player");
    local maxXp = UnitXPMax("player");

    if (level == MAX_PLAYER_LEVEL) then
        C_ChatInfo.SendAddonMessage(SEND_MAX_LEVEL_PREFIX, name, "PARTY");
        return;
    end

    local exhaustionStateID, exhaustionStateName, exhaustionStateMultiplier = GetRestState();

    gxp_debug("exhaustionStateID", exhaustionStateID);
    gxp_debug("exhaustionStateName", exhaustionStateName);
    gxp_debug("exhaustionStateMultiplier", exhaustionStateMultiplier);

    exhaustionStateID = exhaustionStateID or 3
    exhaustionStateName = exhaustionStateName or ""
    exhaustionStateMultiplier = exhaustionStateMultiplier or 0

    local message = name..";"..englishClass..";"..exhaustionStateID..";"..exhaustionStateName..";"..exhaustionStateMultiplier..";"..level..";"..xp..";"..maxXp;
    
    gxp_debug("SendAddonMessage", message);
    C_ChatInfo.SendAddonMessage(SEND_DATA_PREFIX, message, "PARTY");

    GroupXP_UpdatePlayerData();
end

function GroupXP_UpdatePlayerData()
    local level = UnitLevel("player");
    
    if (level == MAX_PLAYER_LEVEL) then
        return;
    end

    local name = UnitName("player");
    local _, englishClass = UnitClass("player");
    local level = UnitLevel("player");
    local xp = UnitXP("player");
    local maxXp = UnitXPMax("player");

    local exhaustionStateID, exhaustionStateName, exhaustionStateMultiplier = GetRestState();

    local playerData = {name = name, englishClass = englishClass, exhaustionStateID = exhaustionStateID, exhaustionStateName = exhaustionStateName, exhaustionStateMultiplier = exhaustionStateMultiplier, level = level, xp = xp, maxXp = maxXp};
    GroupXP_DrawData(0, playerData);
end

function GroupXP_ReceiveData(prefix, message, channel, sender)
    if (prefix == nil) then
        return;
    elseif (prefix == SEND_MAX_LEVEL_PREFIX) then
        groupData[message] = nil;
    elseif (prefix == SEND_DATA_PREFIX) then
        gxp_debug("GroupXP_ReceiveData", message);
    
        local name, englishClass, exhaustionStateID, exhaustionStateName, str_exhaustionStateMultiplier, str_level, str_xp, str_maxXp = strsplit(";", message, 8);
    
        local playerName, _ = UnitName("player");
    
        if (sender == playerName) then
            return;
        end
    
        local exhaustionStateMultiplier = tonumber(str_exhaustionStateMultiplier);
        local level = tonumber(str_level);
        local xp = tonumber(str_xp);
        local maxXp = tonumber(str_maxXp);
    
        local groupData = groupData or {};
        groupData[name] = {name = name, englishClass = englishClass, exhaustionStateID = exhaustionStateID, exhaustionStateName = exhaustionStateName, exhaustionStateMultiplier = exhaustionStateMultiplier, level = level, xp = xp, maxXp = maxXp};
     end   

     GroupXP_UpdateUI();
end

function GroupXP_UpdateUI()
    for index, id in pairs(PARTY_UNITS) do
        local unitData = groupData[UnitName(id)];
        if (UnitExists(id) and unitData ~= nil) then
            GroupXP_DrawData(index, unitData);
        else
            GroupXP_RemoveData(index);
        end
    end
end

function GroupXP_DrawData(index, data)
    gxp_debug("GroupXP_DrawData", index);

    local frame = _G["GroupXPFrame"..index];
    if (not frame) then
        frame = CreateFrame("Frame", "GroupXPFrame"..index, StatusTrackingBarManager);
        frame:SetParent(StatusTrackingBarManager);
        frame:SetFrameStrata("TOOLTIP");
        frame:SetSize(9, 9);

        frame:EnableMouse(true);
        frame:HookScript("OnEnter", GroupXP_ShowTooltip);
        frame:HookScript("OnLeave", GroupXP_HideTooltip);      
    end

    CleanUI_RemoveTextures(frame);


    local texture = frame:CreateTexture("GroupXPFrameTexture"..index, "OVERLAY");
    texture:SetTexture("Interface\\Addons\\GroupXP\\pin");
    texture:SetDrawLayer("OVERLAY");

    texture:ClearAllPoints();
    texture:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0);
    texture:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0);


    local classColor = RAID_CLASS_COLORS[data.englishClass];
    texture:SetDesaturated(1);
    texture:SetVertexColor(classColor.r, classColor.g, classColor.b);

    texture:Show();

    frame.data = data;

    local parent = StatusTrackingBarManager.SingleBarLarge;

    if (StatusTrackingBarManager.SingleBarLarge:IsVisible()) then
        parent = StatusTrackingBarManager.SingleBarLarge;
    end

    if (data.maxXp and data.maxXp > 0) then

        local width = 0;
        for i, bar in ipairs(StatusTrackingBarManager.bars) do
            if (not parent) then
                parent = bar.StatusBar;
            end
    
            width = bar.StatusBar:GetWidth();
        end

        local xOffset = (parent:GetWidth() - width) / 2;

        local posX = xOffset + data.xp / data.maxXp * width;
    
        frame:ClearAllPoints();
        frame:SetPoint("BOTTOM", parent, "BOTTOMLEFT", posX, 3);
        frame:Show();
    else
        frame:Hide();
    end
end

function GroupXP_RemoveData(index)
    gxp_debug("GroupXP_RemoveData");

    local frame = _G["GroupXPFrame"..index];
    if (frame) then
        frame.data = nil;
        frame:Hide();
    end
end

function GroupXP_CheckData()
    local data = {}

    for index, id in pairs(PARTY_UNITS) do
        local unitData = groupData[UnitName(id)];
        if (unitData) then
            data[UnitName(id)] = unitData
        end
    end

    groupData = data;
    GroupXP_UpdateUI();
end

function GroupXP_ShowTooltip(self)
    gxp_debug("GroupXP_ShowTooltip");

    if (self.data == nil) then
        return;
    end

    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    
    GameTooltip:SetText(self.data.name, 1.0, 1.0, 1.0);

    GameTooltip:AddDoubleLine("|c333399ffLevel|r", "|cffffffff"..self.data.level.."|r");

    GameTooltip:AddLine(" ");
    local percent = math.ceil(self.data.xp / self.data.maxXp * 100);
    local xpText = format(XP_TEXT, BreakUpLargeNumbers(self.data.xp), BreakUpLargeNumbers(self.data.maxXp), percent );
    GameTooltip:AddLine("|cffffffff"..xpText.."|r");

    GameTooltip:AddLine("|cffffffff"..format(EXHAUST_TOOLTIP1, self.data.exhaustionStateName, (self.data.exhaustionStateMultiplier * 100)).."|r");

    GameTooltip:Show();
end

function GroupXP_HideTooltip()
    GameTooltip:FadeOut();
end